function sarray = AddStruct2StructArray(sarray,s)
%  function sarray = AddStruct2StructArray(sarray,s)
%  Add fields of a struct as last member of an array of structs
%  inputs:
%     sarray: an array of structs (can be empty)
%     s: the structure whose fields will be added
%  outputs:
%     sarray: the updated struct array
%
%  example:
%  sarr = struct([]);
%  stats = makeD2s(....
%  sarr = AddStruct2StructArray(sarr,stats);
%  stats = makeD2s(....
%  sarr = AddStruct2StructArray(sarr,stats);
%  >> sarr
%  
%  sarr = 
%  
%  1x2 struct array with fields:
%      d2ideal
%      d2NQ
%    ...
%  REA 27/Mar/2013 13:55
%  COPYRIGHT:  Robert E. Alvarez, 2013. All rights reserved.
  
assert(isstruct(sarray) && isstruct(s) );
fnames = fieldnames(s);
sarray(end+1).(fnames{1}) = s.(fnames{1});

if length(fnames) > 1
   for k = 2:length(fnames)
      sarray(end).(fnames{k}) = s.(fnames{k});
   end
end
