function  [bins,fhat] = HistPlot(dat,varargin) 
% function  [bins,fhat] = HistPlot(dat,varargin) 
% computes the histogram of dat with optimal bin width 
% plots on the current figure (or subplot)
% input:
%  dat: a 1D vector with the data
%  optional:
%    do_label_axes: ( = false) label the axes with 'Density' and 'Data'
%   barcolor: ( = [0.333333 0 0.666667](blue)) color for bars. Can also be std matlab color e.g.'black'
%   facecolor: ( none) color for bars. Can also be std matlab color e.g.'black'
%    linewidth: (= 1) bar linewidth
%   plot_frac: (optional = false if not present) If present, plots rather the relative fraction rather than number of cases per bin 
% outputs:
%   bins: histogram bins
%   fhat: histogram data normalized so sum = 1
% REA 8/9/05 - 2/11/09

% use code created by dfittool but modified to make standalone function
nreqargs = 1;
assert(nargin>=nreqargs);
do_label_axes = false;
barcolor = [0.333333 0 0.666667];
facecolor = 'none';
linewidth = 1;
plot_frac = false;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'do_label_axes';          do_label_axes=true;
     case 'plot_frac';          plot_frac=true;
     case 'barcolor';        barcolor =varargin{i+1};     i=i+1;
     case 'facecolor';        facecolor =varargin{i+1};     i=i+1;
     case 'linewidth';         linewidth =varargin{i+1};     i=i+1;
    otherwise
        error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end


% Force all inputs to be column vectors
dat = double(dat(:));

% Set up figure to receive datasets and fits
f_ = gcf; %clf;
%figure(f_);
set(f_,'Units','Pixels');
% set(f_,'Units','Pixels','Position',[414 529.5 680 478.45]);
legh_ = []; legt_ = {};   % handles and text for legend
ax_ = newplot;
%set(ax_,'Box','on');
hold on;

% --- Plot data originally in dataset "dat data"
t_ = ~isnan(dat);
Data_ = dat(t_);
[F_,X_] = ecdf(Data_,'Function','cdf'...
              );  % compute empirical cdf
Bin_.rule = 1;
[C_,E_] = dfswitchyard('dfhistbins',Data_,[],[],Bin_,F_,X_);
[N_,C_] = ecdfhist(F_,X_,'edges',E_); % empirical pdf from cdf
if plot_frac
   N_ = N_/sum(N_);
end
h_bar = bar(C_,N_,'hist');
set(h_bar,'FaceColor',facecolor,'EdgeColor',barcolor,...
       'LineStyle','-', 'LineWidth',linewidth);
if do_label_axes
  xlabel('Data');
  ylabel('Density')
  legh_(end+1) = h_bar;
  legt_{end+1} = 'dat data';
end % if do_label_axes

% Nudge axis limits beyond data limits
xlim_ = get(ax_,'XLim');
if all(isfinite(xlim_))
   xlim_ = xlim_ + [-1 1] * 0.01 * diff(xlim_);
   set(ax_,'XLim',xlim_)
end

if nargout
    bins = C_;
    fhat = N_;
end