function indexes = IntersectSegPolyline(seg,pline)
% function indexes = IntersectSegPolyline(seg,pline)
% returns indexes of intersection of line segment seg with polyline pline
% line segments are represented as 1x2 complex numbers corresponding to the node points
% the polyline pline is a complex array of the node points (length>=2)
%  function demonstrates itself with a random polyline if called without args
% REA 7/8/04

if nargin < 1
        help(mfilename)
        z1 = 10;
	n = z1;
	z1 = sort(rand(1, n)) + sqrt(-1)*rand(1, n);
	z2 = rand(1, 2) + sqrt(-1)*rand(1, 2);
	hold off
	plot(real(z1), imag(z1), 'g', ...
			real(z2), imag(z2), 'r')
	tic
	result = feval(mfilename, z2,z1);
	disp([' ## Elapsed time: ' int2str(round(toc)) ' s'])
	if any(result)
		zi = interp1(1:length(z1), z1, result(result ~= 0));
		hold on
		plot(real(zi), imag(zi), 'bo')
	end
	hold off
	title([mfilename ' ' int2str(n)])
	figure(gcf)
	try, zoomsafe, catch, end
	return
end
seg = seg(:); % column vectors
pline = pline(:); %column vector
assert(length(seg)==2);
assert(length(pline)>=2);
result = zeros(length(pline)-1, 1);
% use formula from 'Geometric Tools for Computer Graphics' by P.J. Schneider and D.H. Eberly p. 242
% see also my lab notebook for 7/8 - 7/9/04
% convert to notation corresponding to my equations: test line seg is P1 + s*d1 and each polyline seg is P2 + t*d2
pline = [pline(1:end-1) diff(pline)]; 
seg = [seg(1),seg(2)-seg(1)];

eps = 1.0e-6;
c1 = repmat(seg,size(pline,1),1);
P1 = c1(:,1);
d1 = c1(:,2);
P2 = pline(:,1);
d2 = pline(:,2);
D21 = P2 - P1;
perp_d2_d1 = imag(conj(d2).*d1);
test = abs(perp_d2_d1) > eps*sqrt(abs(d2).*abs(d2));
n1 = imag(conj(d2).*D21);
n2 = imag(conj(d1).*D21);
npnts = length(perp_d2_d1);
s = zeros(npnts,1);
t = s;
for i = 1:npnts %can probably vectorize this
   if(test(i))
     s(i) = n1(i)/perp_d2_d1(i);
     t(i) = n2(i)/perp_d2_d1(i);
   else % lines are parallel so no intersect
     s(i) = -1;
     t(i) = -1;
   end
end % for i = 1:length(perp_d2_d1)
%s = imag(conj(d2).*D21)./perp_d2_d1;
%t = imag(conj(d1).*D21)./perp_d2_d1;
[hits,index_s,index_t] = intersect(find((s>=0)&(s<=1)),find((t>=0)&(t<=1)));
if(~isempty(hits))
   result(hits,1) = hits + t(hits);
end;
if nargout > 0
	indexes = result;
else
	disp(result)
end
