% P55Neural.m Neural net estimator?
% 15/Apr/2015 13:25
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2015. All rights reserved.
% To run the code, extract the zip file to a directory, cd to that directory
% delete the following code section since it is only for my computer
% all the functions required for the code should be in the zip file

%% init vars--delete this section if not on 
post_title = 'P55Neural';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
    % set path to paper code to access Atable estimator function
p = genpath('L:\Projects\DEsolvePaper\');
addpath(p);
% MakeBlogPostFileLists(post_title);

%% Create the spectrum
tube_voltage = 100; % kV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');

%% mu data-- mus are mass attenuation coeffs
materials = {'C5O2H8','Al'}; % acrylic plastic and aluminum
specdat.mus=  [xraymu(materials{1},specdat.egys), xraymu(materials{2},specdat.egys)];
materials_names = {'Plastic','AL'};
nbasis = size(specdat.mus,2);

%% make the  geometry
thicknesses_max = [20 1.5]; % max thicknesses for object gm/cm^2 
   % define end points of lines in A-space
   % these are at different angles so they are different materials
fracs =  [ ...
    .25 .6;  ...
    .8 .25; ...
    0.8*ones(1,nbasis); ...
    ];
r2s = bsxfun(@times,fracs,thicknesses_max);
nlines = size(r2s,1);       
npoints_on_line=  25;% number of points along each line
   % make lines thru A-space
Alines = linspaceNDim(zeros(size(r2s)),r2s,npoints_on_line);   

%% Set threshold energies for PHA bins
 % find nearest specdat.egys indexes to threshold energies in Zimmerman paper
specdat.idx_threshold =  interp1(specdat.egys, 1:numel(specdat.egys),[40,50,60,70],'nearest','extrap');
nbins = numel(specdat.idx_threshold) + 1; % do not use 25 keV threshold since my bins start from 0
disp(specdat.egys(specdat.idx_threshold)')

%% make the calibration phantom
calib_thicknesses_max = thicknesses_max; % range of step wedge thicknesses to span the object
ncalib_steps = 15; % number of steps for step wedges
t1s = geospace(0,calib_thicknesses_max(1),ncalib_steps);  
t2s =  geospace(0,calib_thicknesses_max(2),ncalib_steps); 
[X,Y] = meshgrid(t1s,t2s);
cdat.As = [X(:),Y(:)];
cdat.ns = PoissonLambda(cdat.As, specdat);
   % divide by 0 thickness value 
assert(all(cdat.As(1,:)==0));  
cdat.ns_zero = cdat.ns(1,:);
cdat.log_ns_zero = log(cdat.ns_zero);
cdat.logI = -bsxfun(@minus,log(cdat.ns), cdat.log_ns_zero);

%% get estimates from random data for points on 3 lines through A-space
    % set up the spectrum
nphotons = 2e6;
specdat.specnum = nphotons*specdat.specnum_norm; 
   % Monte Carlo parameters
rng('default'); % set random number gen to default state so get same results as in paper figures   
nrand_trials = 2000; % number of random trials per point
idx_test = repmat((1:npoints_on_line)',nrand_trials,1); % duplicate points on line for extract data in random trials

   % estimators to compare
estimators2use = {'NeuralNetEstimator','AtableSolveEquations'};
nestimators = length(estimators2use);
   % initialize the estimator parameters
solvedatAtable = [];
solvedatNeural.net = @nnet17Apr15;  % this is pre-built function specific to parameters
  % for this run. In general, need to re-compute the net with code in NeuralNetEstimator.m
 
sdats = struct([]); % struct array to hold results for each estimator and line
for kline = 1:nlines
    fprintf('start line %d\n',kline);
       % get the Avectors of the line
    As_line_actual = squeeze(Alines(kline,:,:))';
        % generate random data
    lambdas = PoissonLambda(As_line_actual,specdat); % expected values
    nphotons_expected_values = repmat(lambdas,nrand_trials,1); 
    ns_random = poissrnd(nphotons_expected_values); 
    tfs_zero = (ns_random(:)==0);
    if any(tfs_zero)
        warning('some random counts are zero');
        fprintf('kline %d frac of zeros %g\n',kline,sum(tfs_zero)/numel(ns_random));
        ns_random(tfs_zero) = nphotons_expected_values(tfs_zero); % force no zeros so can take log
    end
       % prep the test data struct
    tdat_noise.As = repmat(As_line_actual,nrand_trials,1); 
       
       % compute noisy logI data from random photon counts ns_random
       % in following, use fact that first point of line is for zero thickness
    tdat_noise.logI = -bsxfun(@minus, log(ns_random), log(lambdas(1,:)));
    tdat_no_noise.As = tdat_noise.As; % copy As then add logI with no noise
    tdat_no_noise.logI = -bsxfun(@minus, log(nphotons_expected_values), log(lambdas(1,:)));
        % calculate covariance for Atable estimator
    cdat.RL = diag(1./nphotons_expected_values( round(0.3*npoints_on_line) ,:));
    for kestimator = 1:nestimators
        estimator2use = estimators2use{kestimator};
        switch estimator2use
            case 'NeuralNetEstimator';
                % use the pre-defined solvedatNeural struct to save some time
                % computing the neural net. Pre-defined only works with
                % specific parameters for this case. In general, re-compute
                % by leaving out the ,'solvedat',solvedatNeural parameters in the following
            dat_noise = NeuralNetEstimator(cdat,tdat_noise,'solvedat',solvedatNeural);
            dat_no_noise = NeuralNetEstimator(cdat,tdat_no_noise,'solvedat',solvedatNeural);
            case 'AtableSolveEquations';
                if isempty(solvedatAtable)
                    dat_noise = AtableSolveEquations(cdat,tdat_noise);
                    solvedatAtable = dat_noise.solvedat;
                    dat_no_noise = AtableSolveEquations(cdat,tdat_no_noise,'solvedat',solvedatAtable);
                else
                    dat_noise =  AtableSolveEquations(cdat,tdat_noise,'solvedat',solvedatAtable);
                    dat_no_noise = AtableSolveEquations(cdat,tdat_no_noise,'solvedat',solvedatAtable);
                end
            otherwise
                error('Estimator %s not found',estimator2use);
        end
        s.dat_noise = dat_noise;
        s.dat_no_noise = dat_no_noise;
            % add result to summary struct array
        sdats = AddStruct2StructArray(sdats,s);
    end
end
  % reshape sdats array for easier access
sdats = reshape(sdats,nestimators,nlines);  

%% display the A-plane lines with no-noise and noisy estimates
colors = 'rbg';
plot_symbols = '+*d';
fh = figure;
zlabels = zeros(nlines,1); % pre-allocate locations for line number labels
estimate_types = {'dat_no_noise','dat_noise'};
for ktype = 1:length(estimate_types)
    type = char(estimate_types{ktype});
    for kestimator = 1:nestimators
        subplot(2,2,nestimators*(ktype-1)+(kestimator-1)+1)
        if ktype==1,title(estimators2use{kestimator},'fontsize',14);end
        hold on
        for kline = 1:nlines
               % first plot the actual line
            zs = zcomp(squeeze(Alines(kline,:,:))');
            color = colors(kline);
            symbol = plot_symbols(kline);
            plot(zs,['-' color],'linewidth',1.5);
               % get locations for line number labels beyond end of line
            zlabels(kline) = interp1(1:npoints_on_line,zs,npoints_on_line+2, ...
                'linear','extrap');        
                % plot the no-noise estimates. Use only the first trial for each point since no noise
            dat = sdats(kestimator,kline).(type);
%             dat = sdats(sub2ind([nestimators nlines],kestimator,kline)).(type);
            zAs_estimate = zcomp(dat.Astest(1:npoints_on_line,:)); 
            plot(zAs_estimate,[symbol color],'markersize',5);
        end
        hold off
            % label the lines
        h_label = labelPointsZ(gca, zlabels);
            % and the estimate type
        h_type = text(13,.7,type,'fontsize',13,'interpreter','none');
        set(gca,'fontsize',12,'linewidth',1.5);
        xlabel('A_1 (gm/cm^2)','fontsize',13);
        if kestimator==1,ylabel('A_2 (gm/cm^2)','fontsize',13);end
           % force no negative scales
        v = axis;
        axis([0 v(2) 0 v(4)])
    end
end
set(fh,'color','white','position',[200 200 900 900])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'AplaneLines';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% extract the estimates' variance, MSE, and normalized std. dev. of synthesized mu
  % variables to hold results
CR_var_all = zeros(nlines,npoints_on_line,nbasis,nestimators);
means_all = zeros(nlines,npoints_on_line,nbasis,nestimators);
var_all = zeros(nlines,npoints_on_line,nbasis,nestimators); % Monte Carlo variance
mse_all = zeros(nlines,npoints_on_line,nbasis,nestimators); % Monte Carlo mean squared error
   % parameters for 70 keV computed attentuation coefficient test
   % specdat.idx_threshold(end) is index for 70 keV
   % multiply by fracs (see line definitions in geometry cell above) 
   % to get components of mus of material for each line in A-space
mus_70 = bsxfun(@times,fracs,specdat.mus(specdat.idx_threshold(end),:));
zmus_70 = zcomp(mus_70); % convert to complex notation so can use my zdot function
mu_normalized_stds = zeros(nlines,npoints_on_line,nestimators); % normalized std dev of mus
for kestimator = 1:nestimators
    for kline = 1:nlines
        dat_noise = sdats(kestimator,kline).dat_noise;
              % process the random estimates for each point in the line
             % prep for CRLB calc
        As_line_actual = squeeze(Alines(kline,:,:))';
            % get expected values for PHA bin counts
        lambdas = PoissonLambda(As_line_actual,specdat); % expected values
        nphotons_expected_values = repmat(lambdas,nrand_trials,1); 
        Ms_all = mubarNbinNdim(specdat,As_line_actual);
        for kt = 1:npoints_on_line
                    % first the mean, variance and mean squared errors of the data
               idx = find(kt==idx_test); % index to random estimates for this point on line
               As4point = dat_noise.Astest(idx,:); % random A vectors for this point on line
               means_all(kline,kt,:,kestimator) = mean(As4point); % means at each point
               var_all(kline,kt,:,kestimator) = var(As4point); % variance
               mse_all(kline,kt,:,kestimator) = mean(dat_noise.Asdiff(idx,:).^2); % mean squared error
                  % then the CR bound -- use constant covariance approximation   
              M = Ms_all(:,:,kt);
              lambdas_pnt = nphotons_expected_values(kt,:);
              RLinv = diag(lambdas_pnt);
              RA_CR = inv(M'*RLinv*M);
              CR_var_all(kline,kt,:,kestimator) = diag(RA_CR)'; 
                % compute the Zimmerman-Schmidt mu noise parameter
              if kt>1   % can't compute Zimmerman-Schmidt mu for first point on line with 0 thickness
                     % dot product with mus_70 to get the effective mus for material
                     % in following use zdot because dot function can't be used with bsxfun
                     % since Matlab dot does not accept matrix of vectors, only single vectors
                  mus4point = bsxfun(@zdot,zcomp(As4point),zmus_70(kline));
                  mu_normalized_stds(kline,kt,kestimator) = std(mus4point)/ ...
                      dot(As_line_actual(kt,:),mus_70(kline,:));
              end
        end
    end
end

%% make a scatterplot of variances and CRLB or std devs vs distance from center
fh = figure;
set(fh,'color','white','position',[100 100 700 900])
plot_symbols = {'d','*','o'};
colors = 'rbg';
A_names = cell(1,nbasis);
for kmatl = 1:nbasis
    A_names{kmatl} = sprintf('A_{%s}',materials_names{kmatl});
end
hax = zeros(nbasis);
h_plots = [];
error_type = 'MSE'; % 'variance'; %
p = 1; % subplot position
for kline = 1:nlines
    thicknesses = sqrt(sum((squeeze(Alines(kline,:,:))').^2,2));
    for kbasis = 1:nbasis
        color = colors(kline);
        subplot(nlines,nbasis,p)
        p = p + 1;
        CR_var = squeeze(CR_var_all(kline,:,:,kestimator)); 
        h_plots{end+1} = semilogy(thicknesses,CR_var(:,kbasis),['-' color],'linewidth',1.5);
        errs = []; %prep to set range
        hold on 
        for kestimator = 1:nestimators
            switch lower(error_type)
                case 'mse';
                 mse_dat = squeeze(mse_all(kline,:,:,kestimator));            
                 h_plots{end+1} = semilogy(thicknesses,mse_dat(:,kbasis), ...
                     [plot_symbols{kestimator} color]);
                 errs = [errs; mse_dat(:,kbasis)];
                case 'variance';
                 var_dat = squeeze(var_all(kline,:,:,kestimator));
                 h_plots{end+1} = semilogy(thicknesses,var_dat(:,kbasis), ...
                     [plot_symbols{kestimator} color]);
                 errs = [errs; var_dat(:,kbasis)];
                case 'std'
                    std_dat = squeeze(mu_normalized_stds(kline,:,kestimator));
                    if kbasis==1
                        h_plots{end+1} = plot(thicknesses,std_dat, ...
                            [plot_symbols{kestimator} color]);
                    end
                otherwise
                    error('unrecognized error type');
            end
        end
        hold off
        [mn,mx] = range2([errs(:);CR_var(:,kbasis)]);
        axis([0 max(thicknesses) 10^floor(log10(mn)) 10^ceil(log10(mx))]);
        hax(kbasis) = gca;
        set(gca,'fontsize',13','linewidth',1.5)
        if kbasis==1 && kline==3
            xlabel('A vector magnitude (gm/cm^2)','fontsize',13);
        end
        if kbasis==1 && kline==2
            ylabel(sprintf('%s (gm/cm^2)^2',error_type),'fontsize',13);
        end
        if kline==1
            title(A_names{kbasis},'fontsize',13);
        end
        if kline==2 && kbasis==2
            legend('CRLB','neural','Atable','location','southeast')
        end
        if kbasis == 1
             text(0.1*max(thicknesses),10^ceil(log10(mx)-1),sprintf('line %d',kline),'fontsize',13);
        end
    end
end

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
if strcmp(error_type,'MSE')
    fname = 'MSEvsCRLB3lines';
else
    fname = 'VARvsCRLB3lines';
end
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% display histograms of neural net calibrator fit errors
fh = figure;
set(fh,'color','white','position',[200 200 650 450])
for kbasis = 1:nbasis
    subplot(1,nbasis,kbasis);
    er = sdats(1,1).dat_no_noise.solvedat.As_fit_err(:,kbasis);
    HistPlot(er,'barcolor','k', ...
        'facecolor',colors(kbasis),'plot_frac');
    xlabel('fit errors','fontsize',12);
    if kbasis == 1
        ylabel('frac of occurrences','fontsize',12);
    end
    title(sprintf('A_%d',kbasis),'fontsize',14)
       % put std dev into plot
    v = axis;
    text(v(1)+0.5*(v(2)-v(1)), v(3)+0.8*(v(4)-v(3)), ...
        sprintf('std.dev.\n%5.2g',std(er)),'fontsize',12); 
    set(gca,'linewidth',1.5,'fontsize',12);
    fprintf('kbasis %d std(errs) %g\n',kbasis,std(er));
end

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'Fit_error_histogram';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% draw optimal energy bins
   % create high egy-resolution data so do not get slanted line in mu-plot
egyshr = fliplr(linspace(min(specdat.egys),max(specdat.egys),1000)); % high res energies
mu_ref = MusFromMaterials('soft_tissue',egyshr); 

fh = figure;
set(fh,'color','white','position',[ 100 100 580 375])
plot(specdat.egys, 100*specdat.specnum_norm/max(specdat.specnum_norm),'-k','linewidth',1.5);
axis([0 max(specdat.egys) 0.1 100]);
xlabel('energy (keV)','fontsize',12);
Esall = [1; specdat.egys(specdat.idx_threshold); max(specdat.egys)];
ys = [100; 100];
harr = zeros(5,1);
for kbin = 1:nbins
    xs = [Esall(kbin); Esall(kbin+1)];
    zs = zcomp([xs ys]);
    hl = line([xs(1) xs(1)],[0.1 100],'LineStyle','--','linewidth',1);
    if kbin==5
        text(0.98*mean(xs),10,num2str(kbin),'fontsize',12);
    else
        text(mean(xs),10,num2str(kbin),'fontsize',12)
    end
    harr(kbin) = zarrow2head(zs(1),zs(2),'color','r');
    psn = get(harr(kbin),'position');
    set(harr(kbin),'position',[psn(1) 0.3 psn(3:end)]);
end
set(gca,'fontsize',12','linewidth',1.5);

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'PHAbins';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot the normalized std devs vs distance from center
fh = figure;
set(fh,'color','white','position',[100 100 350 800])
plot_symbols = {'d','*','o'};
colors = 'rbg';
fontsiz = 13;
A_names = cell(1,nbasis);
for kmatl = 1:nbasis
    A_names{kmatl} = sprintf('A_{%s}',materials_names{kmatl});
end
hax = zeros(nbasis);
h_plots = [];
error_type = 'MSE'; %'variance'; % 
p = 1; % subplot position
for kline = 1:nlines
    thicknesses = sqrt(sum((squeeze(Alines(kline,:,:))').^2,2));
    color = colors(kline);
    subplot(nlines,1,kline)
    hold on 
    for kestimator = 1:nestimators
        std_dat = squeeze(mu_normalized_stds(kline,:,kestimator));
        h_plots(end+1) = plot(thicknesses(2:end),std_dat(2:end),[plot_symbols{kestimator} color]);
    end
    hold off
    set(gca,'fontsize',fontsiz','linewidth',1.5,'yscale','log')
    v = axis;
    text(0.3*max(thicknesses),0.8*v(4),sprintf('line %d',kline), ...
        'color',color,'fontsize',fontsiz);

    if kline==1
        title('normalized std. dev.','fontsize',fontsiz);
    end
    if kline==2
        ylabel('normalized std. dev.','fontsize',fontsiz);
        legend('neural','Atable','location','best')
    end
    if kline==3
        xlabel('A vector magnitude (gm/cm^2)','fontsize',fontsiz);
    end
end

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'StdDev';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',4,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% save matlab network graphics description
%{
code commented out. 
If want to use it, you need to run
the neural net toolbox nnstart GUI, create the net using 
parameters in NeuralNetEstimator.m and export the net
Of course, delete these lines also
  % convert from java to Matlab graphics
jframe = view(net);
fh = figure('Menubar','none', 'Position',[100 100 565 166]);
jpanel = get(jframe,'ContentPane');
[~,h] = javacomponent(jpanel);
set(h, 'units','normalized', 'position',[0 0 1 1])
   %# close java window
jframe.setVisible(false);
jframe.dispose();
    % save the graphics to disk
figno = sprintf('-f%d',fh);
fname = 'netgraph';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi
%}
